package com.sd.business.controller.order;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.core.domain.BaseSelectDto;
import com.sd.business.entity.order.dto.OrderInfoDto;
import com.sd.business.entity.order.dto.OrderInfoSelectDto;
import com.sd.business.entity.order.dto.SkuSpecPackageBomDto;
import com.sd.business.entity.order.dto.SkuSpecPriceDto;
import com.sd.business.entity.order.enums.OrderExceptionTypeEnum;
import com.sd.business.entity.order.vo.OrderInfoVo;
import com.sd.business.entity.order.vo.OrderPackageBomVo;
import com.sd.business.entity.order.vo.OutBomVo;
import com.sd.business.entity.order.vo.SkuSpecPriceVo;
import com.sd.business.service.order.OrderInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;


/**
 * <p>
 * 订单 前端控制器
 * </p>
 *
 * @author
 * @since 2023-12-11
 */
@RestController
@RequestMapping("/orderInfo")
public class OrderInfoController {

    @Autowired
    private OrderInfoService orderInfoService;

    /**
     * 订单分页
     */
    @PostMapping("/page")
    public Page<OrderInfoVo> page(@RequestBody OrderInfoSelectDto dto) {
        return orderInfoService.getPage(dto);
    }

    /**
     * 订单明细
     */
    @PostMapping("/detail")
    public OrderInfoVo detail(@RequestBody BaseSelectDto dto) {
        return orderInfoService.detail(dto.getId());
    }

    /**
     * 订单新增
     */
    @PostMapping("/add")
    public void add(@RequestBody OrderInfoDto dto) {
        orderInfoService.add(dto);
    }

    /**
     * 订单编辑
     */
    @PostMapping("/edit")
    public void edit(@RequestBody OrderInfoDto dto) {
        orderInfoService.edit(dto);
    }

    /**
     * 订单删除
     */
    @PostMapping("/delete")
    public void delete(@RequestBody BaseSelectDto dto) {
        orderInfoService.deleteValidated(dto.getId());
    }

    /**
     * 订单删除并回滚库存
     */
    @PostMapping("/deleteAndStore")
    public void deleteAndStore(@RequestBody BaseSelectDto dto) {
        orderInfoService.deleteAndStore(dto.getId());
    }

    /**
     * 订单确认
     */
    @PostMapping("/confirmation")
    public void confirmation(@RequestBody OrderInfoDto dto) {
        orderInfoService.confirmation(dto);
    }

    /**
     * 通过sku规格id获取产品价格
     */
    @PostMapping("/getSkuSpecPrice")
    public SkuSpecPriceVo getSkuSpecPrice(@Validated @RequestBody SkuSpecPriceDto dto) {
        return orderInfoService.getSkuSpecPrice(dto);
    }

    /**
     * 获取订单异常类型
     */
    @PostMapping("/getExceptionTypeList")
    public List<Map<String, Object>> getExceptionTypeList() {
        return OrderExceptionTypeEnum.getMapList();
    }

    /**
     * 转为正常订单
     */
    @PostMapping("/turnToRegularOrder")
    public void turnToRegularOrder(@RequestBody BaseSelectDto dto) {
        orderInfoService.turnToRegularOrder(dto.getId());
    }

    /**
     * 获取订单送货单号
     */
    @PostMapping("/getOrderDeliveryCode")
    public String getOrderDeliveryCode() {
        return orderInfoService.getOrderDeliveryCode();
    }

    /**
     * 保存订单送货单号
     */
    @PostMapping("/saveOrderDeliveryCode")
    public void saveOrderDeliveryCode(@RequestBody OrderInfoDto dto) {
        orderInfoService.saveOrderDeliveryCode(dto);
    }

    /**
     * 挂起订单
     */
    @PostMapping("/suspendOrder")
    public void suspendOrder(@RequestBody BaseSelectDto dto) {
        orderInfoService.suspendOrder(dto.getId());
    }

    /**
     * 取消挂起订单
     */
    @PostMapping("/cancelSuspendOrder")
    public void cancelSuspendOrder(@RequestBody BaseSelectDto dto) {
        orderInfoService.cancelSuspendOrder(dto.getId());
    }

    /**
     * 订单删除退料清单
     */
    @PostMapping("/returnBomList")
    public List<OutBomVo> returnBomList(@RequestBody BaseSelectDto dto) {
        return orderInfoService.returnBomList(dto.getId());
    }

    /**
     * 修改订单产品包装
     */
    @PostMapping("/editOrderPackageBom")
    public void editOrderPackageBom(@RequestBody OrderInfoDto dto) {
        orderInfoService.editOrderPackageBom(dto);
    }

    /**
     * 获取产品包装bom列表
     */
    @PostMapping("/getSkuSpecPackageBomList")
    public List<OrderPackageBomVo> getSkuSpecPackageBomList(@Validated @RequestBody SkuSpecPackageBomDto dto) {
        return orderInfoService.getSkuSpecPackageBomList(dto);
    }

    /**
     * 订单导出
     */
    @PostMapping("/orderExportExcel")
    public void orderExportExcel(@RequestBody BaseSelectDto dto) {
        orderInfoService.orderExportExcel(dto.getId());
    }

}
