package com.sd.business.service.order;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.core.service.BaseService;
import com.sd.business.entity.order.dto.OrderInfoDto;
import com.sd.business.entity.order.dto.OrderInfoSelectDto;
import com.sd.business.entity.order.dto.SkuSpecPackageBomDto;
import com.sd.business.entity.order.dto.SkuSpecPriceDto;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.order.vo.OrderInfoVo;
import com.sd.business.entity.order.vo.OrderPackageBomVo;
import com.sd.business.entity.order.vo.OutBomVo;
import com.sd.business.entity.order.vo.SkuSpecPriceVo;

import java.util.List;


/**
 * <p>
 * 订单 服务类
 * </p>
 *
 * @author
 * @since 2023-12-11
 */
public interface OrderInfoService extends BaseService<OrderInfo> {

    /**
     * 订单分页
     */
    Page<OrderInfoVo> getPage(OrderInfoSelectDto dto);

    /**
     * 订单明细
     */
    OrderInfoVo detail(Long id);

    /**
     * 订单新增
     */
    void add(OrderInfoDto dto);

    /**
     * 订单编辑
     */
    void edit(OrderInfoDto dto);

    /**
     * 订单删除
     */
    void delete(Long id);

    /**
     * 删除订单并回滚库存
     */
    void deleteAndStore(Long id);

    /**
     * 验证订单后删除
     */
    void deleteValidated(Long id);

    /**
     * 订单确认
     */
    void confirmation(OrderInfoDto dto);

    /**
     * 通过sku规格id获取产品价格
     */
    SkuSpecPriceVo getSkuSpecPrice(SkuSpecPriceDto dto);

    /**
     * 转为正常订单
     */
    void turnToRegularOrder(Long id);

    /**
     * 获取订单送货单号
     */
    String getOrderDeliveryCode();

    /**
     * 保存订单送货单号
     */
    void saveOrderDeliveryCode(OrderInfoDto dto);

    /**
     * 挂起订单
     */
    void suspendOrder(Long id);

    /**
     * 取消挂起订单
     */
    void cancelSuspendOrder(Long id);

    /**
     * 订单删除退料清单
     */
    List<OutBomVo> returnBomList(Long id);

    /**
     * 修改订单产品包装
     */
    void editOrderPackageBom(OrderInfoDto dto);

    /**
     * 获取产品包装bom列表
     */
    List<OrderPackageBomVo> getSkuSpecPackageBomList(SkuSpecPackageBomDto dto);

    /**
     * 订单导出
     */
    void orderExportExcel(Long id);

}
