package com.sd.business.service.production.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.exception.ServiceException;
import com.sd.business.entity.artwork.po.ArtworkLibrary;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.order.po.OrderSku;
import com.sd.business.entity.production.dto.ProductionWorkOrderDto;
import com.sd.business.entity.production.dto.ProductionWorkOrderSelectDto;
import com.sd.business.entity.production.po.ProductionWorkOrder;
import com.sd.business.entity.production.vo.ProductionWorkOrderVo;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.mapper.production.ProductionWorkOrderMapper;
import com.sd.business.service.artwork.ArtworkLibraryService;
import com.sd.business.service.order.OrderInfoService;
import com.sd.business.service.order.OrderSkuService;
import com.sd.business.service.production.ProductionWorkOrderService;
import com.sd.business.service.sku.SkuSpecService;
import com.sd.framework.util.Assert;
import com.sd.framework.util.sql.Sql;
import com.sd.mq.config.WorkOrderConfig;
import com.sd.mq.entity.WorkOrderDetail;
import com.sd.mq.entity.WorkOrderMessage;
import com.sd.mq.util.RabbitMqUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;


/**
 * <p>
 * 生产工单 服务实现类
 * </p>
 *
 * @author
 * @since 2023-12-11
 */
@Service
public class ProductionWorkOrderServiceImpl extends ServiceImpl<ProductionWorkOrderMapper, ProductionWorkOrder> implements ProductionWorkOrderService {

    @Autowired
    private OrderInfoService orderInfoService;

    @Autowired
    private OrderSkuService orderSkuService;

    @Autowired
    private SkuSpecService skuSpecService;

    @Autowired
    private ArtworkLibraryService artworkLibraryService;

    @Override
    public Page<ProductionWorkOrderVo> getPage(ProductionWorkOrderSelectDto dto) {

        Page<ProductionWorkOrderVo> page = Sql.create(ProductionWorkOrderVo.class)
                .selectAll(ProductionWorkOrder.class)
                .from(ProductionWorkOrder.class)
                .orderByDesc(ProductionWorkOrder::getId)
                .page(dto);

        return page;
    }

    @Override
    public ProductionWorkOrderVo detail(Long id) {

        ProductionWorkOrderVo vo = Sql.create(ProductionWorkOrderVo.class)
                .selectAll(ProductionWorkOrder.class)
                .from(ProductionWorkOrder.class)
                .eq(ProductionWorkOrder::getId, id)
                .one();

        Assert.notNull(vo, "未知数据");

        return vo;
    }

    @Override
    public void add(ProductionWorkOrderDto dto) {
        save(dto);
    }

    @Override
    public void edit(ProductionWorkOrderDto dto) {
        updateById(dto);
    }

    @Override
    public void delete(Long id) {
        removeById(id);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void addByOrderId(Long orderId) {
        OrderInfo orderInfo = orderInfoService.getById(orderId);
        if (orderInfo == null) {
            throw new ServiceException("未知订单");
        }

        List<ProductionWorkOrder> productionWorkOrderList = new ArrayList<>();

        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.eq(OrderSku::getOrderId, orderId));

        String code = ObjectUtil.defaultIfNull(orderInfo.getWlnCode(), orderInfo.getCode());

        int num = 1;

        for (OrderSku orderSku : orderSkuList) {
            BigDecimal quantity = orderSku.getQuantity();
            for (int i = 0; i < quantity.intValue(); i++) {
                ProductionWorkOrder productionWorkOrder = new ProductionWorkOrder();
                productionWorkOrder.setType(1);
                productionWorkOrder.setCode(code + String.format("-%05d", num));
                productionWorkOrder.setOrderId(orderId);
                productionWorkOrder.setOrderSkuId(orderSku.getId());
                productionWorkOrder.setSkuSpecId(orderSku.getSkuSpecId());
                productionWorkOrder.setBomSpecId(orderSku.getBomSpecId());
                productionWorkOrder.setPrintType(orderSku.getPrintType());
                productionWorkOrder.setStatus(0);
                productionWorkOrderList.add(productionWorkOrder);
                num++;
            }
        }

        skuSpecService.attributeAssign(productionWorkOrderList, ProductionWorkOrder::getSkuSpecId, (item, skuSpec) -> {
            item.setMachinedPanel(skuSpec.getMachinedPanel());
        });

        saveBatch(productionWorkOrderList);

        sendMq(productionWorkOrderList);
    }

    @Override
    public void sendMq(List<ProductionWorkOrder> list) {
        if (list.isEmpty()) {
            return;
        }

        // 订单详情
        List<Long> orderSkuIdList = list.stream().map(ProductionWorkOrder::getOrderSkuId).collect(Collectors.toList());
        Map<Long, OrderSku> orderSkuMap = orderSkuService.byIdsToMap(orderSkuIdList);

        // 图稿库
        List<Long> artworkLibraryIdList = orderSkuMap.values().stream()
                .map(OrderSku::getArtworkLibraryId)
                .filter(Objects::nonNull)
                .filter(item -> item != 0)
                .distinct()
                .collect(Collectors.toList());
        Map<Long, ArtworkLibrary> artworkLibraryMap;
        if (artworkLibraryIdList.isEmpty()) {
            artworkLibraryMap = Collections.emptyMap();
        } else {
            artworkLibraryMap = artworkLibraryService.byIdsToMap(artworkLibraryIdList);
        }

        // sku规格
        List<Long> skuSpecIdList = list.stream().map(ProductionWorkOrder::getSkuSpecId).collect(Collectors.toList());
        Map<Long, SkuSpec> skuSpecMap = skuSpecService.byIdsToMap(skuSpecIdList);

        List<WorkOrderDetail> workOrderDetailList = list.stream().map(item -> {
            WorkOrderDetail workOrderDetail = new WorkOrderDetail();

            workOrderDetail.setWorkOrderId(item.getId());

            Long orderSkuId = item.getOrderSkuId();
            OrderSku orderSku = orderSkuMap.get(orderSkuId);
            Long artworkLibraryId = orderSku.getArtworkLibraryId();

            if (artworkLibraryId != null && artworkLibraryId != 0) {
                ArtworkLibrary artworkLibrary = artworkLibraryMap.get(artworkLibraryId);
                if (artworkLibrary != null) {
                    workOrderDetail.setImgPath("artwork\\" + artworkLibrary.getArtworkName() + "\\" + artworkLibrary.getImgName());
                    workOrderDetail.setFilePath("artwork\\" + artworkLibrary.getArtworkName() + "\\" + artworkLibrary.getFileName());
                }
            }

            if (StrUtil.isAllBlank(workOrderDetail.getImgPath(), workOrderDetail.getFilePath())) {
                workOrderDetail.setImgPath("tempArtwork\\" + orderSku.getBlueprint().replace("https://os.winfaster.cn/sd/", "")
                        .replace("test/", "")
                        .replace("prod/", "")
                        .replace("dev/", ""));
                workOrderDetail.setFilePath("tempArtwork\\" + orderSku.getProductionDocument().replace("https://os.winfaster.cn/sd/", "")
                        .replace("test/", "")
                        .replace("prod/", "")
                        .replace("dev/", ""));
            }

            SkuSpec skuSpec = skuSpecMap.get(item.getSkuSpecId());

            workOrderDetail.setLength(skuSpec.getLength());
            workOrderDetail.setWide(skuSpec.getWidth());
            workOrderDetail.setHight(skuSpec.getHeight());

            return workOrderDetail;
        }).collect(Collectors.toList());

        WorkOrderMessage workOrderMessage = new WorkOrderMessage();
        workOrderMessage.setWorkOrderDetailList(workOrderDetailList);

        RabbitMqUtil.send(WorkOrderConfig.DIRECT_EXCHANGE_NAME, WorkOrderConfig.ADD_WORK_ORDER_QUEUE_NAME, workOrderMessage);

    }

}
